%% HOUSING BOOM-BUST CYCLES AND ASYMMETRIC MACROPRUDENTIAL POLICY
% DYNARE CODE FOR THE MODEL UNDER THE REFERENCE REGIME, i.e. HIGH DELTA
%  William Gatt
%  22 November 2017
%  First version: 22 November 2017
%  Updated for JMCB: 8 June 2021





% Declare variables and shocks
     @#include "variables_and_shocks.mod"

% Calibrate model 
     @#include "param_calib.mod"
 
% Declare the model equations
    @#include "model_equations.mod"

% Add relevant equations to model      
    model;
       [name = 'Notional debt']
       B_NOT = (1-gamma)*(m*N_H_W) + gamma*B_NOT(-1)/PI;
       [name = 'Net housing wealth -- borrowers']
       N_H_W = ((q_1*PI_1)/R)*Hb; 
       [name = 'Actual debt']
       B = B_NOT;
       [name = 'Leverage limit']
       MaxLev = B - B_NOT;

       [name = 'Macroprudential policy rule']
       m = (m_ss^(1-rho_m))*(((B_4Y)/STEADY_STATE(B_4Y))^((-delta_mH)*(1-rho_m)))*(m(-1)^(rho_m));     
%        m = (m_ss^(1-rho_m))*(((B_4Y)/STEADY_STATE(B_4Y))^((-delta_mH)*(1-rho_m)))*(m(-1)^(rho_m))*m_shock;       
    end;

% Initialise SS solver
    @#include "model_initval_ss.mod"

% Calibrate shocks
    @#include "shock_calibration.mod"

options_.noprint=1;
steady(solve_algo=4); 

check; 
% model_diagnostics;

stoch_simul(order=1,nocorr,nomoments,irf=0,noprint,nograph);

% stoch_simul(order=1,nocorr,nomoments,irf=20,nograph) Y PI4 q Cb LTI B_4Y r4 m mju j PI_Y4;

% exo_list = {'eps_ETA_surprise'};
% endo_r_list = {'R','r4','r','rr','rr4','m','psi','mju',...
%                'Hs','Hb','B_4Y','LTI'};
% plot_IRFS;
