%% HOUSING BOOM-BUST CYCLES AND ASYMMETRIC MACROPRUDENTIAL POLICY
% 
% Loads and generates IRFs from Dynare structures
%
% William Gatt
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% SS VALUES
    for j=1:length(oo_.steady_state)
        temp= M_.endo_names{j};
        eval(['SS.' temp,' = oo_.steady_state(',num2str(j),');']);
    end

%% Generating structure from list of shocks

n_shocks = length(exo_list);
    for m = 1:n_shocks
         struct.shocks.(exo_list{m})=(exo_list{m});
    end

%% Generating structure from list of endogenous variables

endo_list = cellstr(var_list_);
n_endovar = length(endo_list);
    for n = 1:n_endovar
        struct.evars.(endo_list{n})=(endo_list{n});
    end

%% Copying in IRFs from DYNARE, and expressing them in relative deviations 
if exist('GLOBAL_IRFS')==1  
    clear GLOBAL_IRFS       % Removing this matrix if already exists; changing IRF horizon and running script again causes a dimension mismatch
end

    for n = 1:n_endovar
        for m = 1:n_shocks
            thisVar = strcat(endo_list{n},'_');
            thisVar1 = strcat(thisVar,exo_list{m});
            Steady_State = SS.(endo_list{n});         %The steady state value of the variable
            
            if Steady_State == 0  && max(strcmpi(endo_list(n),endo_r_list))==1   % Ratio variables
                struct.irfs.(endo_list{n}).(exo_list{m}) = (oo_.irfs.(eval('thisVar1')))*100';
                GLOBAL_IRFS(:,n) = (oo_.irfs.(eval('thisVar1')))'; 
            elseif Steady_State == 1  && max(strcmpi(endo_list(n),endo_r_list))==1   % Gross ratio variables
                struct.irfs.(endo_list{n}).(exo_list{m}) = (oo_.irfs.(eval('thisVar1')));
                GLOBAL_IRFS(:,n) = (oo_.irfs.(eval('thisVar1')))*100';
            elseif max(strcmpi(endo_list(n),endo_r_list))==1 || Steady_State == 0   % Ratio variables OR variables with SS = 0 (such as adjustment costs)
                struct.irfs.(endo_list{n}).(exo_list{m}) = (oo_.irfs.(eval('thisVar1')))';
                GLOBAL_IRFS(:,n) = (oo_.irfs.(eval('thisVar1')))'; 
            else
                struct.irfs.(endo_list{n}).(exo_list{m}) = (oo_.irfs.(eval('thisVar1'))./Steady_State)*100';          
                GLOBAL_IRFS(:,n) = (oo_.irfs.(eval('thisVar1'))./Steady_State)*100';
            end
            clear thisVar thisVar1
        end
    end