%% HOUSING BOOM-BUST CYCLES AND ASYMMETRIC MACROPRUDENTIAL POLICY
% model equations

model;
% SAVERS

    [name = 'Budget constraint (savers)']
    Cs + B  + q*(Hs - Hs(-1)) = R(-1)*B(-1)/PI + ws*Ns + (1-MC)*Y;
    
    [name = 'Euler equation (Deposit supply -- savers)']
    ((1-g)/Cstilde) = betas*(R/PI(+1))*((1-g)/Cstilde(+1));

    [name = 'Labour supply (savers)']
    ws*((1-g)/Cstilde) = tau*(Ns^(varphi));
    
    [name = 'Housing demand (savers)']
    q*((1-g)/Cstilde) = j*Hs^(-1) + betas*(q(+1)*((1-g)/Cstilde(+1)));
    
    [name = 'Habits']
    Cstilde = Cs - g*Cs(-1);

% BORROWERS
    
    [name = 'Budget constraint']
    Cb + (R(-1)*B(-1))/PI + q*(Hb - Hb(-1)) = wb*Nb + B ;
    
    [name = 'Euler equation (Loan demand -- borrowers)']
    (1-g)/Cbtilde = betab*(R/PI(+1))*((1-g)/Cbtilde(+1)) + R*mju;
      
    [name = 'Labour supply (borrowers)']
    wb*((1-g)/Cbtilde) = tau*Nb^(varphi);
    
    [name = 'Housing demand (borrowers)']
      q*((1-g)/Cbtilde) = (j)/Hb + betab*q(+1)*((1-g)/Cbtilde(+1)) + mju*(m*q_1*PI_1);
    
    [name = 'Habits (borrowers)']
    Cbtilde = Cb - g*Cb(-1);
    
% INTERM. FIRMS    
    
    [name = 'MPL - savers']
    ws = alphaa*MC*(Y/Ns);
    
    [name = 'MPL - borrowers']
    wb = (1-alphaa)*MC*(Y/Nb);

    [name = 'MC']
    MC = ((ws/(alphaa))^(alphaa))*((wb/((1-alphaa)))^((1-alphaa)));
    
    [name = 'Total labour input']
    N = (Ns^(alphaa))*(Nb^(1-alphaa));
    
    [name = 'Price setting']
    Upsilon = ((sigma-1)/sigma)*ptilde*Phi;
     
    [name = 'Price setting - Upsilon']
    Upsilon = (Y*MC/Cstilde) + omega*betas*(PI(+1)^(sigma))*Upsilon(+1);
    
    [name = 'Price setting - Phi']
    Phi     = (Y/Cstilde) + omega*betas*(PI(+1)^(sigma-1))*Phi(+1);
    
    [name = 'Price dispersion cost']
    ptilde = ((1-omega)/(1-omega*PI^(sigma-1)))^(1/(sigma-1));
   
    [name = 'Price dispersion']
    s = (1-omega)*(ptilde)^(-sigma) + omega*(PI^sigma)*s(-1);   

% MARKET CLEARING
    
    [name = 'Housing Market Market Clearing']
    Hs + Hb = 1;
      
    
% POLICY

    [name = 'Monetary policy']
    R = (((1/betas) *((PI)^(delta_Pi))*(Y/STEADY_STATE(Y))^delta_Y)^(1-rho_R))*(R(-1)^(rho_R));

 
% SHOCK PROCESSES
       
    [name = 'Housing demand shock process']
    log(j) = (1-rho_j)*log(j_ss) + rho_j*log(j(-1)) + eps_j + nshock + eps_j44;      

    [name = 'News - contemporaneous']
    eps_j40 = eps_j4;
    
    [name = 'News - 1st lag']
    eps_j41 = eps_j40(-1);
    
    [name = 'News - 2nd lag']
    eps_j42 = eps_j41(-1);
    
    [name = 'News - 3rd lag']
    eps_j43 = eps_j42(-1);
    
    [name = 'News - 4th lag']
    eps_j44 = eps_j43(-1);
    
    [name = 'Unrealised News shock']
    nshock = eps_ETA_surprise + eps_ETA4 ;       

    [name = 'Unrealised News -- contemporaneous']
    eps_ETA0 = eps_ETA;
    
    [name = 'Unrealised News -- 1st lag']
    eps_ETA1 = eps_ETA0(-1);
    
    [name = 'Unrealised News -- 2nd lag']
    eps_ETA2 = eps_ETA1(-1);
    
    [name = 'Unrealised News -- 3rd lag']
    eps_ETA3 = eps_ETA2(-1);
    
    [name = 'Unrealised News - 4th lag']
    eps_ETA4 = eps_ETA3(-1);
    
% GREAT RATIOS AND OTHER DEFINITIONS
    [name = 'Share of consumption - savers']
    Cs_Y     = Cs/Y;  
    
    [name = 'Share of consumption - borrowers']
    Cb_Y     = Cb/Y;  
    
    [name = 'Total housing wealth to GDP']
    qH_4Y    = q/(4*Y);   
    
    [name = 'Housing wealth to GDP - savers']
    qHs_4Y   = (q*Hs)/(4*Y);
    
    [name = 'Housing wealth to GDP - borrowers']
    qHb_4Y   = (q*Hb)/(4*Y);
    
    [name = 'Credit to GDP (annualised)']
    B_4Y     = B/(4*Y);     
    
    [name = 'Leverage - i.e. external vs internal sources of finance']
    LTI = B/(wb*Nb*4);
    
    [name = 'Annualised nominal rate']
    r4      = (R-1)*400; 
    
    [name = 'Quarterly nominal interest rate']
    r       = (R-1)*100;
    
    [name = 'Quarterly real interest rate']
    rr      = (R/PI(+1)-1)*100;
    
    [name = 'Annualised real interest rate']
    rr4     = (R/PI(+1)-1)*400; 
    
    [name = 'Real house price growth']
    PI_q     = (q/q(-1));    
    
    [name = 'Credit growth (year-on-year)']
    PI_4B    = (B/B(-1))*4;       
    
    [name = 'YoY inflation']
    PI4     =   PI*PI1*PI2*PI3;
    
    [name = 'YoY inflation - 1st lag']
    PI1     =   PI(-1);
    
    [name = 'YoY inflation - 2nd lag']
    PI2     =   PI1(-1);
    
    [name = 'YoY inflation - 3rd lag']
    PI3     =   PI2(-1);
    
    [name = 'YoY GDP growth']
    PI_Y     =   (Y/Y(-1));
    
    [name = 'YoY GDP growth - 1st lag ']
    PI_Y1     =   PI_Y(-1);
    
    [name = 'YoY GDP growth - 2nd lag']
    PI_Y2     =   PI_Y1(-1);
    
    [name = 'YoY GDP growth - 3rd lag']
    PI_Y3     =   PI_Y2(-1);
    
    [name = 'YoY inflation']
    PI_Y4     =   PI_Y*PI_Y1*PI_Y2*PI_Y3;
    
 


    [name = 'Labour income -- borrowers']
    wNb     = wb*Nb;
    
    [name = 'Labour income -- savers']
    wNs     = ws*Ns;
    
    % AUXILIARY VARIABLES
    [name = 'Lead on house prices']
    q_1  = q(+1);
    
    [name = 'Lead on inflation']
    PI_1 = PI(+1);
    
end;