%% set inputs for solution below
%  The program produces responses to the shocks selected below under
%  irfshock. Paths for all the endogenous variables in the model selected
%  are produced. VariableName_difference holds the piece-wise linear solution for
%  VariableName.  VariableName_uncdifference holds the linear solution for
%  VariableName.

% clear all ; 
% clc;
% close all

global M_ oo_ dyn_version

    % Model regimes
    regime_11       = 'Mod_borrconst_binding_high'; % constraint BINDS, Credit BOOM
    regime_01       = 'Mod_borrconst_slack_high';   % constraint SLACK, Credit BOOM
    regime_10       = 'Mod_borrconst_binding_low';  % constraint BINDS, Credit BUST
    regime_00       = 'Mod_borrconst_slack_low';    % constraint SLACK, Credit BUST
    
    
    constraint1          =   'mju<-mju_ss';     % escape from reference model - constraint becomes slack
    constraint_relax1    =   'MaxLev > 0';      % back to reference model     - constraint becomes binding
    constraint2          =   'B_4Y < -0.0001';  % escape from reference model - constraint becomes slack
    constraint_relax2    =   'B_4Y > -0.0001';  % back to reference model     - constraint becomes binding

%% EXTRA CODE
%     tic;
    
%     % Pick innovation for IRFs
%     irfshock        = char('eps_ETA','eps_ETA_surprise','eps_j4','eps_j');  % bubble shock (shocks 1 and 2), news shock (shock 3), unanticipated shock (shock 4)
%     [nshocks,yyy]   = size(irfshock);
%     irfperiods      = 20;
%     nperiods        = irfperiods+200;
%     shockssequence  = zeros(nperiods,nshocks);     
%     
%     % Calibrating shock size
%     shockstd    = [0.054]; % 1 standard deviation shock --> 1% change in q
% 
%     shockON     = [ 1  1  0  0 ];   
% %     shock_sign   = 1;
% %     shock_size  = 1;          %standard deviations
% 
%     % Positive shock
%         shockssequence(1,1)     =  shockstd*shockON(1)*shock_sign*shock_size;        % Bubble shock - boom
%         shockssequence(1+4,2)   =  shockstd*shockON(2)*(-shock_sign)*shock_size;     % Bubble shock - bust
%         shockssequence(1,3)     =  shockstd*shockON(3)*shock_sign*shock_size;        % News shock
%         shockssequence(4:8,4)   =  shockstd*shockON(4)*shock_sign*shock_size;        % Unanticipated shock

%     % Negative shock
%         shockssequence(50,1)    = shockstd(1)*shockON(1)*shocksign;
%         shockssequence(50+4,2)  = shockstd(2)*shockON(2)*shocksign;
%         shockssequence(120,3)    = shockstd(3)*shockON(3)*-shocksign;
        
        
%     irfshock        =   char('eps_j4');  % label for innovation for IRFs    
%     [nshocks,yyy]   =   size(irfshock);
% 
%     irfperiods      =   40;
%     nperiods        =   irfperiods+80;
%     shockssequence  =   zeros(nperiods,nshocks); 
%     shockstd        =   0.054;                 % 1% increase in house prices on impact
% 
%     shockssequence(1,1)    =  shockstd;     % Positive shock for 4 periods

%%
fprintf('\n============================================================== \n\n');
fprintf('Solving model using OCCBIN \n - Model with OBC in borrowing and asymmetric LTV rule \n\n')
fprintf('==============================================================\n\n');

% pause(2);
    
% [zdatalinear zdatapiecewise zdatass oobase_ Mbase_  ] = ...
%   solve_one_constraint(regime,regimestar,...
%   constraint, constraint_relax,...
%   shockssequence,irfshock,nperiods);

maxiterr = 100;

% [zdatalinear zdatapiecewise zdatass oobase_ Mbase_ conv_errs] = solve_two_constraints(...
[zdatalinear zdatapiecewise zdatass oobase_ Mbase_ , conv_errs reg_count] = solve_two_constraints1(...
  regime_11,regime_01,regime_10,regime_00,...
  constraint1, constraint2,...
  constraint_relax1, constraint_relax2,...
  shockssequence,irfshock,nperiods,0,200,0,DynParse1);


    fprintf('\nSimulation complete! ... extracting IRFs...\n');

    % unpack the IRFs (compatible with Dynare 4.6+)
    for i=1:Mbase_.endo_nbr
      eval([Mbase_.endo_names{i,:},'_l=zdatalinear(:,i);']);
      eval([Mbase_.endo_names{i,:},'_p=zdatapiecewise(:,i);']);
      eval([Mbase_.endo_names{i,:},'_ss=zdatass(i);']);
    end

    % old code compatible with Dynare 4.4 -
%     for i=1:Mbase_.endo_nbr
%       eval([deblank(Mbase_.endo_names(i,:)),'_l=zdatalinear(:,i);']);
%       eval([deblank(Mbase_.endo_names(i,:)),'_p=zdatapiecewise(:,i);']);
%       eval([deblank(Mbase_.endo_names(i,:)),'_ss=zdatass(i);']);
%     end
                       
    
%     fprintf(' \nBorrowing constraint is slack in %2.0f quarters\n', sum(mju_p/(-mju_ss)>0.999));  %when constraint is slack, mju_p is *almost* equal to -mju_ss
%     
%     etime=toc;
%     fprintf('...total simulation time: %4.2f seconds\n', etime);


%%
return
set(0,'DefaultLineLineWidth',2);

figure;
subplot(2,4,1); grid on; hold on;
h3=plot(q_p(1:irfperiods)/q_ss*100);
h4=plot(q_l(1:irfperiods)/q_ss*100); %axis tight;
title('$q$','interpreter','latex','Fontsize',20);

subplot(2,4,2); grid on; hold on;
h1=plot(Cb_p(1:irfperiods)/Cb_ss*100);
h2=plot(Cb_l(1:irfperiods)/Cb_ss*100);
title('$C_{b}$','interpreter','latex','Fontsize',20);

subplot(2,4,3); grid on; hold on;
h9=plot(Y_p(1:irfperiods)/Y_ss*100);
h10=plot(Y_l(1:irfperiods)/Y_ss*100);
title('$Y$','interpreter','latex','Fontsize',20);

subplot(2,4,4); grid on; hold on;
h5=plot(B_4Y_p(1:irfperiods)+B_4Y_ss);
h6=plot(B_4Y_l(1:irfperiods)+B_4Y_ss); %axis tight;
title('$B4Y$','interpreter','latex','Fontsize',20);

subplot(2,4,5); grid on; hold on;
h7=plot(mju_p(1:irfperiods)+mju_ss);
h8=plot(mju_l(1:irfperiods)+mju_ss); %axis tight;
title('$\mu$','interpreter','latex','Fontsize',20);
legend({'Piecewise linear','Linear'},'Fontsize',12,'Location','SouthEast');

subplot(2,4,6); grid on; hold on;
h9=plot(r4_p(1:irfperiods));
h10=plot(r4_l(1:irfperiods));
title('$R$ (ann.)','interpreter','latex','Fontsize',20);

subplot(2,4,7); grid on; hold on;
h9=plot(m_p(1:irfperiods)+m_ss);
h10=plot(m_l(1:irfperiods)+m_ss);
title('$m$','interpreter','latex','Fontsize',20);

subplot(2,4,8); grid on; hold on;
h9=plot(PI_p(1:irfperiods)/PI_ss*100);
h10=plot(PI_l(1:irfperiods)/PI_ss*100);
title('$\pi$','interpreter','latex','Fontsize',20);
